function [dip] = dipole_fit(dip, dat, sens, vol, constr)

% DIPOLE_FIT performs an equivalent current dipole fit with a single
% or a small number of dipoles to explain an EEG or MEG scalp topography.
%
% Use as
%   [dip] = dipole_fit(dip, dat, sens, vol, constr)

% Copyright (C) 2003, Robert Oostenveld
%
% $Log: dipole_fit.m,v $
% Revision 1.1  2005/11/01 09:34:27  roboos
% replaced EEG/MEG specific dipole fit routines with a single general function
%
% Revision 1.7  2004/02/05 11:55:30  roberto
% fixed clearly apparent bug in error function (would not run)
%
% Revision 1.6  2003/09/12 08:45:41  roberto
% added error if maximum number of iterations is exceeded
%
% Revision 1.5  2003/09/02 13:01:18  roberto
% implemented constrained dipole fitting
%
% Revision 1.4  2003/06/16 10:03:31  roberto
% added check in error function for gui interrupt request (for eeglab)
%
% Revision 1.3  2003/03/13 13:41:00  roberto
% fixed bug in e/meg_error_func in assignment of Nchan
%
% Revision 1.2  2003/03/11 14:45:36  roberto
% updated help and copyrights
%

if nargin<5
  constr = [];
end

% determine whether it is EEG or MEG
if ~isstruct(sens) & size(sens,2)==3
  % definition of electrode positions only, restructure it
  sens.pnt = sens;
  iseeg = 1;
  ismeg = 0;
elseif isfield(sens, 'type') & strcmp(sens.type, 'electrodes')
  iseeg = 1;
  ismeg = 0;
elseif isfield(sens, 'type') & strcmp(sens.type, 'gradiometers')
  iseeg = 0;
  ismeg = 1;
elseif isfield(sens, 'pnt') & ~isfield(sens, 'ori')
  iseeg = 1;
  ismeg = 0;
elseif isfield(sens, 'pnt') &  isfield(sens, 'ori')
  iseeg = 0;
  ismeg = 1;
else
  iseeg = 0;
  ismeg = 0;
end

if ismeg & iseeg
  % this is something that I might implement in the future
  error('simultaneous EEG and MEG not supported');
elseif iseeg
  % ensure that the potential data is average referenced, just like the model potential
  dat = avgref(dat);
end

% reformat the position parameters in case of multiple dipoles
param = reshape(dip.pos', 1, prod(size(dip.pos)));

% reduce the number of parameters to be fitted according to the constraints
if ~isempty(constr)
  param = param(constr.reduce);
end

if exist('fminunc')
  % do non-linear optimization of the dipole parameters using Optimization toolbox fminunc()
  options = optimset('TolFun',1e-9,...
                     'TypicalX',ones(size(param)),...
                     'LargeScale','off',...
                     'MaxIter',100,...
                     'HessUpdate','bfgs',...
                     'Display','iter');
  [param, fval, exitflag, output] = fminunc(@dipfit_error, param, options, dat, sens, vol, constr);
else
  % do non-linear optimization of the dipole parameters using standard Matlab fminsearch()
  options = optimset('MaxIter',3000,...
                     'Display','iter');
  [param, fval, exitflag, output] = fminsearch(@dipfit_error, param, options, dat, sens, vol, constr);
end

if exitflag==0
  error('Maximum number of iterations exceeded before reaching the minimum, I suggest that you try with another initial guess.')
end

% do linear optimization of dipole moment parameters
[err, dip.mom] = dipfit_error(param, dat, sens, vol, constr);

% expand the number of parameters according to the constraints
if ~isempty(constr)
  param = constr.mirror .* param(constr.expand);
end

% reformat the position parameters in case of multiple dipoles
dip.pos = reshape(param, 3, length(param)/3)';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DIPFIT_ERROR computes the error between measured and model data
% and can be used for non-linear fitting of dipole position
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [err, mom] = dipfit_error(param, dat, sens, vol, constr);

% flush pending graphics events, ensure that fitting is interruptible
drawnow;
if ~isempty(get(0, 'currentfigure')) & strcmp(get(gcf, 'tag'), 'stop')
  % interrupt the fitting
  close;
  error('USER ABORT');
end;

% expand the number of parameters according to the constraints
if ~isempty(constr)
  pos = constr.mirror .* param(constr.expand);
else
  pos = param;
end

% this ensures that the position is a Nx3 matrix
pos = reshape(pos, 3, length(pos)/3)';

% construct the leadfield matrix for all dipoles
lf = compute_leadfield(pos, sens, vol);

% compute the optimal dipole moment and the model error
mom = pinv(lf)*dat;
dif = dat - lf*mom;
err = sum(dif(:).^2) / sum(dat(:).^2);
